//
//  CV_Graphics2DBasic.swift
//  Graphics2DTester
//
//  Created by Piotr Fulmański on 03/11/2019.
//  Copyright © 2019 Piotr Fulmański. All rights reserved.
//

// Na strone
// https://developer.apple.com/library/archive/documentation/GraphicsImaging/Conceptual/drawingwithquartz2d/Introduction/Introduction.html
// https://developer.apple.com/library/archive/documentation/Cocoa/Conceptual/CocoaDrawingGuide/Introduction/Introduction.html





// https://www.hackingwithswift.com/articles/103/seven-useful-methods-from-cgrect
// https://www.appcoda.com/bezier-paths-introduction/
// https://blog.thefuntasty.com/swift-stroke-animations-e1b864917255

// http://www.knowstack.com/drawing-in-swift-cocoa/
// https://www.hackingwithswift.com/read/27/3/drawing-into-a-core-graphics-context-with-uigraphicsimagerenderer
// https://www.hackingwithswift.com/quick-start/swiftui/how-to-draw-a-custom-path
// https://www.hackingwithswift.com/example-code/core-graphics/how-to-draw-lines-in-core-graphics-moveto-and-addlineto
// https://www.raywenderlich.com/1101-core-graphics-on-macos-tutorial
// https://medium.com/better-programming/ui-animations-with-swift-2ebb5e6d2292

// https://blog.thefuntasty.com/mvc-without-the-c-what-will-swiftui-change-in-app-architecture-c9ce3f49d256
// https://www.hackingwithswift.com/articles/205/whats-the-difference-between-map-flatmap-and-compactmap
// https://www.packtpub.com/game-development/swift-game-development-third-edition
// https://www.apress.com/gp/book/9781484237175
import Cocoa

@IBDesignable
class CV_Graphics2DBasic: NSView {
    let colorBlack = NSColor(red: 0.0, green: 0.0, blue: 0.0, alpha: 1.0)
    let colorBlue = NSColor.blue
    let colorGreen = NSColor.green
    let colorOrange = NSColor.orange
    let colorRed = NSColor.red
    let colorWhite = NSColor.white
    let colorYellow = NSColor.yellow
    
    override func draw(_ dirtyRect: NSRect) {
        super.draw(dirtyRect)

        testFill(dirtyRect)
        testLine()
        testLineDash()
        testLineWidth()
        createTriangle()
        testLineCompound()
        testRectBorder()
        testRectangleWithRoundedCorners()
        testOvalFill()
        testArc()
        testBezierCubicCurve()
        testGradient()
        testAffineTransform()
    }
    
    private func testFill(_ rect: NSRect) {
        let path = NSBezierPath(rect: rect)
        
        colorYellow.setFill()
        path.fill()
    }
    
    private func testLine() {
        let path = NSBezierPath()
        
        colorBlack.set()
        
        path.lineWidth = 1.0
        
        path.move(to: NSPoint(x: 0, y: 0))
        path.line(to: NSPoint(x: 20, y: 40))
        path.stroke()
    }
    
    private func testLineDash() {
        let path = NSBezierPath()
        let lineDash:[CGFloat] = [5,1,4,1,3,1,2,1,1,1,1,2,1,3,1,4,1,5]
        
        colorBlack.set()
        
        path.lineWidth = 10.0
        // The elements of the lengths parameter specify the widths of the dashes, alternating between the painted and unpainted segments of the line. The phase parameter specifies the starting point of the dash pattern
        path.setLineDash(lineDash,
                         count: 18,
                         phase: 0)
        
        path.move(to: NSPoint(x: 0, y: 160))
        path.line(to: NSPoint(x: 400, y: 160))
        path.stroke()
    }
    
    private func testLineWidth() {
        var path = NSBezierPath()
        var y = 0
        
        colorBlack.set()
        
        path.lineWidth = 0.1
        y = 100
        path.move(to: NSPoint(x: 0, y: y))
        path.line(to: NSPoint(x: 40, y: y))
        path.stroke()
        
        path = NSBezierPath()
        path.lineWidth = 0.5
        y = 110
        path.move(to: NSPoint(x: 0, y: y))
        path.line(to: NSPoint(x: 40, y: y))
        path.stroke()
        
        path = NSBezierPath()
        path.lineWidth = 1.0
        y = 120
        path.move(to: NSPoint(x: 0, y: y))
        path.line(to: NSPoint(x: 40, y: y))
        path.stroke()
        
        path = NSBezierPath()
        path.lineWidth = 1.5
        y = 130
        path.move(to: NSPoint(x: 0, y: y))
        path.line(to: NSPoint(x: 40, y: y))
        path.stroke()
        
        path = NSBezierPath()
        path.lineWidth = 2.0
        y = 140
        path.move(to: NSPoint(x: 0, y: y))
        path.line(to: NSPoint(x: 40, y: y))
        path.stroke()
    }
    
    private func createTriangle() {
        let path = NSBezierPath()
        
        colorBlack.set()
        
        path.lineWidth = 1.0
        path.move(to: NSPoint(x: 60, y: 140))
        path.line(to: NSPoint(x: 120, y: 140))
        path.line(to: NSPoint(x: 90, y: 120))
        path.close()
        
        path.stroke()
    }
    
    private func testLineCompound() {
        let path = NSBezierPath()
        let colorBlack = NSColor(red: 0.0, green: 0.0, blue: 0.0, alpha: 1.0)
        
        colorBlack.set()
        
        path.lineWidth = 1.0
        
        path.move(to: NSPoint(x: 60, y: 100))
        path.line(to: NSPoint(x: 80, y: 100))
        
        path.move(to: NSPoint(x: 100, y: 100))
        path.line(to: NSPoint(x: 120, y: 100))
        path.stroke()
    }
    
    private func testRectBorder() {
        let rect = NSRect(x: 20, y: 40, width: 80, height: 40)
        let path = NSBezierPath(rect: rect)
        
        colorBlack.set()
        path.lineWidth = 1.0
        path.stroke()
    }
    
    private func testRectangleWithRoundedCorners() {
        let rect = NSRect(x: 110,
                          y: 40,
                          width: 80,
                          height: 40)
        let path = NSBezierPath(roundedRect: rect,
                                xRadius: 30,
                                yRadius: 15)
        
        colorBlack.setStroke()
        path.lineWidth = 1.0
        path.stroke()
    }
    
    private func testOvalFill() {
        let rect = NSRect(x: 100,
                          y: 0,
                          width: 40,
                          height: 20)
        let path = NSBezierPath(ovalIn: rect)
        
        colorBlack.setStroke()
        path.lineWidth = 1.0
        path.stroke()
        
        colorOrange.setFill()
        path.fill()
        
        // Compare with
        /*
        colorBlack.set()
        path.lineWidth = 1.0
        path.stroke()
        path.fill()
        */
    }
    
    private func testArc() {
        let path = NSBezierPath()
        
        path.appendArc(withCenter: NSPoint(x: 200, y: 60),
                       radius: 40,
                       startAngle: 0,
                       endAngle: 45,
                       clockwise: false)

        colorBlack.setStroke()
        path.lineWidth = 1.0
        path.stroke()
        
        colorOrange.setFill()
        path.fill()
    }
    
    private func testBezierCubicCurve() {
        let path = NSBezierPath()
        
        path.move(to: NSPoint(x: 220, y: 60))
        path.curve(to: NSPoint(x: 220+50+50+50, y: 60),
                   controlPoint1: NSPoint(x: 220+50, y: 60-60),
                   controlPoint2: NSPoint(x: 220+50+50, y: 60+60))

        colorBlack.setStroke()
        path.lineWidth = 1.0
        path.stroke()
    }
    
    private func testGradient() {
        var path = NSBezierPath(rect: NSRect(x: 120, y: 100, width: 80, height: 40))
        if let gradient = NSGradient(starting: colorWhite,
                                     ending: colorOrange) {
            gradient.draw(in: path, angle: 0)
        }
        path = NSBezierPath(rect: NSRect(x: 220, y: 100, width: 80, height: 40))
        if let gradient = NSGradient(starting: colorWhite,
                                     ending: colorOrange) {
            gradient.draw(in: path, angle: 45)
        }
    }

    private func testAffineTransform() {
        let rect1 = NSRect(x: 50, y: 180, width: 100, height: 50)
        let rect2 = NSRect(x: 70, y: 200, width: 100, height: 50)
        
        let path1 = NSBezierPath(rect: rect1)
        let path2 = NSBezierPath(rect: rect2)

        colorBlack.setStroke()
        path1.lineWidth = 1.0
        path1.stroke()
        
        colorRed.setStroke()
        path2.lineWidth = 1.0
        path2.stroke()
        
        print(rect1.intersects(rect2))
        print(rect1.contains(rect2))
        
        let recIntersection = rect1.intersection(rect2)
        let pathIntersecion = NSBezierPath(rect: recIntersection)

        colorWhite.setFill()
        pathIntersecion.fill()
        
        let rec1Inset = rect1.insetBy(dx: -10, dy: 20)
        let pathRec1Inset = NSBezierPath(rect: rec1Inset)
        
        colorBlue.setStroke()
        pathRec1Inset.lineWidth = 1.0
        pathRec1Inset.stroke()
        
        let rec1Offset = rect1.offsetBy(dx: 100, dy: -50)
        let pathRec1Offset = NSBezierPath(rect: rec1Offset)
        
        colorGreen.setStroke()
        pathRec1Offset.lineWidth = 1.0
        pathRec1Offset.stroke()
        
        var transform = AffineTransform(rotationByDegrees: 5)
        var pathRec1Rotate = NSBezierPath(rect: rect1)
        pathRec1Rotate.transform(using: transform)
        
        colorBlack.setStroke()
        pathRec1Rotate.lineWidth = 1.0
        pathRec1Rotate.stroke()
        
        transform = AffineTransform(rotationByDegrees: 10)
        pathRec1Rotate = NSBezierPath(rect: rect1)
        pathRec1Rotate.transform(using: transform)
        
        colorBlack.setStroke()
        pathRec1Rotate.lineWidth = 1.0
        pathRec1Rotate.stroke()
        
        transform = AffineTransform(rotationByDegrees: 15)
        pathRec1Rotate = NSBezierPath(rect: rect1)
        pathRec1Rotate.transform(using: transform)
        
        colorBlack.setStroke()
        pathRec1Rotate.lineWidth = 1.0
        pathRec1Rotate.stroke()
        
        pathRec1Rotate = NSBezierPath(rect: rect1)
        pathRec1Rotate.transform(using: AffineTransform(translationByX: -50, byY: -180))
        pathRec1Rotate.transform(using: AffineTransform(rotationByDegrees: 15))
        pathRec1Rotate.transform(using: AffineTransform(translationByX: 50, byY: 180))
        
        colorRed.setStroke()
        pathRec1Rotate.lineWidth = 1.0
        pathRec1Rotate.stroke()
    }

}
